@@ -11,11 +11,18 @@ namespace Ui {
 class contentmanagerside;
 }
 
+class KiwixChoiceBox;
+using FilterList = ContentManager::FilterList;
+
 class ContentManagerSide : public QWidget
 {
     Q_OBJECT
 
 public:
+    enum CatalogButtonId {
+        ALL = 0,
+        LOCAL = 1
+    };
     explicit ContentManagerSide(QWidget *parent = 0);
     ~ContentManagerSide();
 
@@ -24,12 +31,15 @@ public:
 private:
     Ui::contentmanagerside *mp_ui;
     ContentManager* mp_contentManager;
-    QCheckBox* mp_languageButton;
-    QListWidget* mp_languageSelector;
-    QCheckBox* mp_categoryButton;
-    QListWidget* mp_categorySelector;
+    KiwixChoiceBox *mp_categories;
+    KiwixChoiceBox *mp_languages;
+    KiwixChoiceBox *mp_contentType;
     QCheckBox* mp_contentTypeButton;
-    QList<ContentTypeFilter*> m_contentTypeFilters;
+    QStringList m_contentTypeFilters;
+
+public slots:
+    void setCategories(QStringList);
+    void setLanguages(ContentManager::LanguageList);
 };
 
 #endif // CONTENTMANAGERSIDE_H
