@@ -3,6 +3,7 @@
 
 #include "library.h"
 #include "contentmanager.h"
+#include "tabbar.h"
 #include "mainwindow.h"
 #include "kiwix/downloader.h"
 #include <kiwix/kiwixserve.h>
@@ -20,6 +21,7 @@
 #include <mutex>
 #include <iostream>
 
+typedef TabBar::TabType TabType;
 
 class KiwixApp : public QtSingleApplication
 {
@@ -32,11 +34,12 @@ public:
         OpenHomePageAction,
         PrintAction,
         NewTabAction,
-        CloseTabAction,
+        CloseCurrentTabAction,
         ReopenClosedTabAction,
         BrowseLibraryAction,
         OpenFileAction,
         OpenRecentAction,
+        OpenMultiZimAction,
         SavePageAsAction,
         SearchArticleAction,
         SearchLibraryAction,
@@ -44,6 +47,9 @@ public:
         ToggleFullscreenAction,
         ToggleTOCAction,
         ToggleReadingListAction,
+        ToggleAddBookmarkAction,
+        ToggleTTSLanguageAction,
+        ToggleTTSVoiceAction,
         ZoomInAction,
         ZoomOutAction,
         ZoomResetAction,
@@ -51,6 +57,9 @@ public:
         PreviousTabAction,
         HistoryBackAction,
         HistoryForwardAction,
+        ReadTextAction,
+        ReadArticleAction,
+        ReadStopAction,
         HelpAction,
         FeedbackAction,
         ReportBugAction,
@@ -59,6 +68,10 @@ public:
         SettingAction,
         DonateAction,
         ExitAction,
+        ExportReadingListAction,
+        ImportReadingListAction,
+        ScrollPreviousTabAction,
+        ScrollNextTabAction,
         MAX_ACTION
     };
 
@@ -75,8 +88,8 @@ public:
     Library* getLibrary() { return &m_library; }
     MainWindow* getMainWindow() { return mp_mainWindow; }
     ContentManager* getContentManager() { return mp_manager; }
-    kiwix::Downloader* getDownloader() { return mp_downloader; }
     TabBar* getTabWidget() { return getMainWindow()->getTabBar(); }
+    SearchBar& getSearchBar() { return getMainWindow()->getTopWidget()->getSearchBar(); }
     QAction* getAction(Actions action);
     QString getLibraryDirectory() { return m_libraryDirectory; };
     kiwix::Server* getLocalServer() { return &m_server; }
@@ -84,18 +97,30 @@ public:
     QString getText(const QString &key) { return m_translation.getText(key); };
     void setMonitorDir(const QString &dir);
     bool isCurrentArticleBookmarked();
+    QString parseStyleFromFile(QString filePath);
+    void saveListOfOpenTabs();
+    void saveWindowState();
+    void saveVoiceName(const QString& langName, const QString& voiceName);
+    void restoreWindowState();
+    void saveCurrentTabIndex();
+    void savePrevSaveDir(const QString& prevSaveDir);
+    QString getSavedVoiceName(const QString& langName) const;
+    QString getPrevSaveDir() const;
+    void restoreTabs();
+    void setupDirectoryMonitoring();
 
 public slots:
+    void newTab();
     void openZimFile(const QString& zimfile="");
     void openUrl(const QString& url, bool newTab=true);
     void openUrl(const QUrl& url, bool newTab=true);
     void printPage();
-    void disableItemsOnLibraryPage(bool displayed);
+    void handleItemsState(TabType);
     void updateNameMapper();
     void printVersions(std::ostream& out = std::cout);
 
 protected:
-    void createAction();
+    void createActions();
     void postInit();
 
 private:
@@ -104,18 +129,18 @@ private:
     KProfile m_profile;
     QString m_libraryDirectory;
     Library m_library;
-    kiwix::Downloader* mp_downloader;
     ContentManager* mp_manager;
     MainWindow* mp_mainWindow;
     QErrorMessage* mp_errorDialog;
-    kiwix::UpdatableNameMapper m_nameMapper;
+    std::shared_ptr<kiwix::UpdatableNameMapper> mp_nameMapper;
     kiwix::Server m_server;
     Translation m_translation;
-    QFileSystemWatcher m_watcher;
+    QSettings* mp_session;
 
     QAction*     mpa_actions[MAX_ACTION];
 
     QString findLibraryDirectory();
+    void loadAndInstallTranslations(QTranslator& translator, const QString& filename, const QString& directory);
 };
 
 QString gt(const QString &key);
