@@ -14,6 +14,8 @@ namespace Ui {
 class MainWindow;
 }
 
+class TableOfContentBar;
+
 class MainWindow : public QMainWindow
 {
     Q_OBJECT
@@ -24,17 +26,22 @@ public:
 
     TabBar*   getTabBar();
     TopWidget* getTopWidget();
+    QWidget getMainView();
+    TableOfContentBar *getTableOfContentBar();
 
 protected:
-    void keyPressEvent(QKeyEvent *event);
     bool eventFilter(QObject* object, QEvent* event) override;
+    void closeEvent(QCloseEvent *event) override;
+    void resizeEvent(QResizeEvent *event) override;
 
 private slots:
     void toggleFullScreen();
-    void when_ReadingList_toggled(bool state);
-    void when_libraryPageDisplayed(bool showed);
+    void tabChanged(TabBar::TabType);
+    void readingListToggled(bool state);
+    void tableOfContentToggled(bool state);
     void hideTabAndTop();
     void showTabAndTop();
+    void updateTabButtons();
 
 private:
     Ui::MainWindow *mp_ui;
