@@ -1,6 +1,9 @@
+class QMenu;
+
 #include "tabbar.h"
 
 #include "kiwixapp.h"
+#include "css_constants.h"
 #include <QAction>
 #include <QTimer>
 #include <QWebEnginePage>
@@ -11,6 +14,15 @@
 #define QUITIFNULL(VIEW) if (nullptr==(VIEW)) { return; }
 #define CURRENTIFNULL(VIEW) if(nullptr==VIEW) { VIEW = currentZimView();}
 
+namespace
+{
+
+QAction* getAction(KiwixApp::Actions action) {
+    return KiwixApp::instance()->getAction(action);
+}
+
+} // unnamed namespace
+
 TabBar::TabBar(QWidget *parent) :
     QTabBar(parent)
 {
@@ -22,78 +34,27 @@ TabBar::TabBar(QWidget *parent) :
     setMovable(true);
     setIconSize(QSize(30, 30));
     connect(this, &QTabBar::currentChanged, this, &TabBar::onCurrentChanged, Qt::QueuedConnection);
-    auto app = KiwixApp::instance();
 
-    connect(app->getAction(KiwixApp::NextTabAction), &QAction::triggered, this, &TabBar::moveToNextTab);
-    connect(app->getAction(KiwixApp::PreviousTabAction), &QAction::triggered, this, &TabBar::moveToPreviousTab);
-    connect(app->getAction(KiwixApp::NewTabAction), &QAction::triggered,
+    connect(getAction(KiwixApp::NextTabAction), &QAction::triggered, this, &TabBar::moveToNextTab);
+    connect(getAction(KiwixApp::PreviousTabAction), &QAction::triggered, this, &TabBar::moveToPreviousTab);
+    connect(getAction(KiwixApp::CloseCurrentTabAction), &QAction::triggered,
             this, [=]() {
-                this->createNewTab(true, false);
-                auto topWidget = KiwixApp::instance()->getMainWindow()->getTopWidget();
-                topWidget->getSearchBar().setFocus(Qt::MouseFocusReason);
-                topWidget->getSearchBar().clear();
-                topWidget->getSearchBar().clearSuggestions();
-                topWidget->getSearchBar().hideSuggestions();
-          });
-    connect(app->getAction(KiwixApp::CloseTabAction), &QAction::triggered,
-            this, [=]() {
-                auto index = this->tabAt(mapFromGlobal(QCursor::pos()));
-                if (index < 0)
-                    return;
-
-                // library tab cannot be closed
-                QWidget *w = mp_stackedWidget->widget(index);
-                if (qobject_cast<ContentManagerView*>(w)) {
-                    return;
-                }
-                this->closeTab(index);
-            });
-    connect(app->getAction(KiwixApp::OpenHomePageAction), &QAction::triggered,
-            this, [=]() {
-                auto current = this->currentWebView();
-                QUITIFNULL(current);
-                current->setUrl("zim://" + current->zimId() + ".zim/");
-            });
-    connect(app->getAction(KiwixApp::SettingAction), &QAction::triggered,
-            this, [=]() {
-                SettingsView* view = KiwixApp::instance()->getSettingsManager()->getView();
-                for (int i = 0 ; i < mp_stackedWidget->count(); i++) {
-                    if (mp_stackedWidget->widget(i) == view) {
-                        setCurrentIndex(i);
-                        return;
-                    }
-                }
-                int index = currentIndex() + 1;
-                mp_stackedWidget->insertWidget(index, view);
-                emit libraryPageDisplayed(false);
-                insertTab(index,QIcon(":/icons/settings.svg"), gt("settings"));
-                QToolButton *tb = new QToolButton(this);
-                tb->setDefaultAction(KiwixApp::instance()->getAction(KiwixApp::CloseTabAction));
-                setTabButton(index, QTabBar::RightSide, tb);
-                setCurrentIndex(index);
+                this->closeTab(currentIndex());
             });
+    connect(getAction(KiwixApp::OpenHomePageAction), &QAction::triggered,
+            this, &TabBar::openHomePage);
+    connect(getAction(KiwixApp::SettingAction), &QAction::triggered,
+            this, &TabBar::openOrSwitchToSettingsTab);
 
     for (int i = 0 ; i <= 9 ; i++) {
         QAction *a = new QAction(this);
-        a->setData(QVariant::fromValue(i));
-        QKeySequence ks(Qt::ALT + (Qt::Key_0 + i));
-        a->setShortcut(ks);
+        a->setShortcut(QKeySequence(Qt::ALT | (Qt::Key_0 + i)));
         addAction(a);
         connect(a, &QAction::triggered, this, [=](){
-            QAction *a = qobject_cast<QAction*>(QObject::sender());
-            if (!a)
-                return;
-
-            bool ok;
-            int tab_n = a->data().toInt(&ok);
-            if (tab_n==0)
-                tab_n=10;
-            if (!ok)
-                return;
-            if (tab_n >= count())
-                return;
-
-            setCurrentIndex(tab_n-1);
+            const int tabIndex = i == 0 ? 9 : i - 1;
+            if (tabIndex < realTabCount()) {
+                setCurrentIndex(tabIndex);
+            }
         });
     }
 
@@ -102,6 +63,30 @@ TabBar::TabBar(QWidget *parent) :
             this, SLOT(onTabMoved(int,int)), Qt::DirectConnection);
 }
 
+void TabBar::openHomePage() {
+    auto current = this->currentWebView();
+    QUITIFNULL(current);
+    current->setUrl("zim://" + current->zimId() + ".zim/");
+}
+
+void TabBar::openOrSwitchToSettingsTab()
+{
+    SettingsView* view = KiwixApp::instance()->getSettingsManager()->getView();
+    for (int i = 0 ; i < mp_stackedWidget->count(); i++) {
+        if (mp_stackedWidget->widget(i) == view) {
+            setCurrentIndex(i);
+            return;
+        }
+    }
+    int index = currentIndex() + 1;
+    mp_stackedWidget->insertWidget(index, view);
+    emit tabDisplayed(TabType::SettingsTab);
+    insertTab(index,QIcon(":/icons/settings.svg"), gt("settings"));
+    setCloseTabButton(index);
+    setCurrentIndex(index);
+    KiwixApp::instance()->saveListOfOpenTabs();
+}
+
 void TabBar::setStackedWidget(QStackedWidget *widget) {
     mp_stackedWidget = widget;
     connect(this, &QTabBar::currentChanged,
@@ -113,14 +98,15 @@ void TabBar::setContentManagerView(ContentManagerView* view)
     qInfo() << "add widget";
     mp_stackedWidget->addWidget(view);
     mp_stackedWidget->show();
-    int idx = addTab(QIcon(":/icons/library-icon.svg"), "");
+    int idx = addTab(QIcon(":/icons/kiwix-logo.svg"), "");
     setTabButton(idx, RightSide, nullptr);
 }
 
-void TabBar::setNewTabButton()
+void TabBar::setNewTabButton(QAction* newTabAction)
 {
     QToolButton *tb = new QToolButton();
-    tb->setDefaultAction(KiwixApp::instance()->getAction(KiwixApp::NewTabAction));
+    tb->setObjectName("newTabButton");
+    tb->setDefaultAction(newTabAction);
     tb->setIcon(QIcon(":/icons/new-tab-icon.svg"));
     int idx = addTab("");
     setTabEnabled(idx, false);
@@ -129,12 +115,12 @@ void TabBar::setNewTabButton()
     setTabButton(idx, QTabBar::RightSide, Q_NULLPTR);
 }
 
+// Returns the count of real tabs with content (excluding the last pseudo-tab
+// that acts as a button for creating new empty tabs; BTW what is the use for
+// such empty tabs?)
 int TabBar::realTabCount() const
 {
-    // The last tab is "+" in TabBar, but that isn't a real tab which displays any content hence the real count is tab count - 1
-    if (count() < 1)
-        return 0;
-    return count() - 1;
+    return count() < 1 ? 0 : count() - 1;
 }
 
 void TabBar::moveToNextTab()
@@ -149,20 +135,45 @@ void TabBar::moveToPreviousTab()
     setCurrentIndex(index <= 0 ? realTabCount() - 1 : index - 1);
 }
 
-ZimView* TabBar::createNewTab(bool setCurrent, bool adjacentToCurrentTab)
+void TabBar::scrollNextTab()
+{
+    const int index = currentIndex();
+    setCurrentIndex(index == realTabCount() - 1 ? index : index + 1);
+}
+
+void TabBar::scrollPreviousTab()
+{
+    const int index = currentIndex();
+    setCurrentIndex(index <= 0 ? index : index - 1);
+}
+
+void TabBar::setCloseTabButton(int index)
+{
+    Q_ASSERT(index > 0 && index < realTabCount());
+
+    QToolButton *tb = new QToolButton(this);
+    tb->setObjectName("closeTabButton");
+    QAction *a = new QAction(QIcon(":/icons/close.svg"), gt("close-tab"), tb);
+    a->setToolTip(getAction(KiwixApp::CloseCurrentTabAction)->toolTip());
+    tb->setDefaultAction(a);
+    setTabButton(index, QTabBar::RightSide, tb);
+    connect(tb, &QToolButton::triggered, this, [=]() {
+        for ( int i = 0; i < realTabCount(); ++i ) {
+            if ( tb == tabButton(i, QTabBar::RightSide) ) {
+                closeTab(i);
+                return;
+            }
+        }
+    });
+}
+
+ZimView* TabBar::createNewTab(bool setCurrent, bool nextToCurrentTab)
 {
     auto tab = new ZimView(this, this);
-    int index;
-    if(adjacentToCurrentTab) {
-        index = currentIndex() + 1;
-    } else {
-        index = realTabCount(); // for New Tab Button
-    }
+    const int index = nextToCurrentTab ? currentIndex() + 1 : realTabCount();
     mp_stackedWidget->insertWidget(index, tab);
-    index = insertTab(index, "");
-    QToolButton *tb = new QToolButton(this);
-    tb->setDefaultAction(KiwixApp::instance()->getAction(KiwixApp::CloseTabAction));
-    setTabButton(index, QTabBar::RightSide, tb);
+    insertTab(index, "");
+    setCloseTabButton(index);
     if (setCurrent) {
         setCurrentIndex(index);
     }
@@ -170,6 +181,7 @@ ZimView* TabBar::createNewTab(bool setCurrent, bool adjacentToCurrentTab)
     connect(tab, &ZimView::webActionEnabledChanged,
             this, &TabBar::onWebviewHistoryActionChanged);
 
+    KiwixApp::instance()->saveListOfOpenTabs();
     return tab;
 }
 
@@ -241,7 +253,7 @@ QSize TabBar::tabSizeHint(int index) const
 {
     QWidget *w = mp_stackedWidget->widget(index);
 
-    if (w && qobject_cast<ContentManagerView*>(w))
+    if ((w && qobject_cast<ContentManagerView*>(w)) || index >= count() - 1)
         return QSize(40, 40); // the "Library" tab is only icon
 
     return QSize(205, 40); // "Settings" and content tabs have text
@@ -263,46 +275,58 @@ void TabBar::triggerWebPageAction(QWebEnginePage::WebAction action, ZimView *wid
 
 void TabBar::closeTabsByZimId(const QString &id)
 {
-    // the last tab is + button, skip it
-    for (int i = count() - 2 ; i >= 0 ; i--) {
+    // 0th tab is always (unless this comment becomes outdated by the time you
+    // read it) the library tab, so iteration could start from 1, however we
+    // shouldn't try to save CPU cycles at the cost of the code breaking
+    // should this comment indeed become outdated ;)
+    for (int i = 0 ; i < realTabCount() ; ++i ) {
         auto *zv = qobject_cast<ZimView*>(mp_stackedWidget->widget(i));
-        if (!zv)
-            continue;
-        if (zv->getWebView()->zimId() == id) {
+        if (zv && zv->getWebView()->zimId() == id) {
             closeTab(i);
         }
     }
 }
 
+QStringList TabBar::getTabUrls() const {
+    QStringList idList;
+    for (int index = 0; index <= mp_stackedWidget->count(); index++)
+    {
+        if (ZimView* zv = qobject_cast<ZimView*>(mp_stackedWidget->widget(index)))
+            idList.push_back(zv->getWebView()->url().url());
+        else if (qobject_cast<SettingsView*>(mp_stackedWidget->widget(index)))
+            idList.push_back("SettingsTab");
+    }
+    return idList;
+}
+
+QStringList TabBar::getTabZimIds() const
+{ 
+    QStringList idList;
+    for (int index = 0; index <= mp_stackedWidget->count(); index++)
+        if (ZimView* zv = qobject_cast<ZimView*>(mp_stackedWidget->widget(index)))
+            idList.push_back(zv->getWebView()->zimId());
+    return idList;
+}
+
 void TabBar::closeTab(int index)
 {
-    // the last tab is + button, cannot be closed
-    if (index == this->realTabCount())
+    // The first and last tabs (i.e. the library tab and the + (new tab) button)
+    // cannot be closed
+    if (index <= 0 || index >= this->realTabCount())
         return;
 
-    setSelectionBehaviorOnRemove(index);
-
-    QWidget *view = mp_stackedWidget->widget(index);
-
-    // library tab cannot be closed
-    if (qobject_cast<ContentManagerView*>(view)) {
-        return;
+    if ( index == currentIndex() ) {
+        setCurrentIndex(index + 1 == realTabCount() ? index - 1 : index + 1);
     }
 
+    QWidget *view = mp_stackedWidget->widget(index);
     mp_stackedWidget->removeWidget(view);
     view->setParent(nullptr);
     removeTab(index);
     view->close();
     view->deleteLater();
-}
 
-void TabBar::setSelectionBehaviorOnRemove(int index)
-{
-    if (index == count() - 2) {
-        setCurrentIndex(index - 1);
-    } else {
-        setCurrentIndex(index + 1);
-    }
+    KiwixApp::instance()->saveListOfOpenTabs();
 }
 
 void TabBar::onCurrentChanged(int index)
@@ -321,18 +345,18 @@ void TabBar::onCurrentChanged(int index)
     if (qobject_cast<SettingsView*>(w)) {
         emit webActionEnabledChanged(QWebEnginePage::Back, false);
         emit webActionEnabledChanged(QWebEnginePage::Forward, false);
-        emit libraryPageDisplayed(false);
+        emit tabDisplayed(TabType::SettingsTab);
         QTimer::singleShot(0, [=](){emit currentTitleChanged("");});
     } else if (auto zv = qobject_cast<ZimView*>(w)) {
         auto view = zv->getWebView();
         emit webActionEnabledChanged(QWebEnginePage::Back, view->isWebActionEnabled(QWebEnginePage::Back));
         emit webActionEnabledChanged(QWebEnginePage::Forward, view->isWebActionEnabled(QWebEnginePage::Forward));
-        emit libraryPageDisplayed(false);
+        emit tabDisplayed(TabType::ZimViewTab);
         QTimer::singleShot(0, [=](){emit currentTitleChanged(view->title());});
     } else if (qobject_cast<ContentManagerView*>(w)) {
         emit webActionEnabledChanged(QWebEnginePage::Back, false);
         emit webActionEnabledChanged(QWebEnginePage::Forward, false);
-        emit libraryPageDisplayed(true);
+        emit tabDisplayed(TabType::LibraryTab);
         QTimer::singleShot(0, [=](){emit currentTitleChanged("");});
     }
     else {
@@ -340,6 +364,8 @@ void TabBar::onCurrentChanged(int index)
         // In the future, other types of tabs can be added.
         // For example, About dialog, or Kiwix Server control panel.
     }
+
+    KiwixApp::instance()->saveCurrentTabIndex();
 }
 
 void TabBar::fullScreenRequested(QWebEngineFullScreenRequest request)
@@ -411,16 +437,28 @@ void TabBar::paintEvent(QPaintEvent *e)
         initStyleOption(&tabopt, i);
 
         bool need_fade_out = false;
+        bool textRightToLeft = tab_title.isRightToLeft();
+        bool appRightToLeft = QWidget::isRightToLeft();
 
+        const int padding = CSS::QTabBar::tab::padding;
         QRect tabTextRect = style()->subElementRect(QStyle::SE_TabBarTabText, &tabopt, this);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+        const int border = CSS::QTabBar::tab::border;
 
+        // Add Padding to left, right. Padding is 4px. Add 5 to account for 
+        // Extra pixel from border.
+        tabTextRect.setX(tabTextRect.x() + padding + border);
+        tabTextRect.setWidth(tabTextRect.width() - padding - border);
+#else
+        // Qt6 correctly adds left and right padding but now incorrectly adds
+        // top or bottom padding.
+        tabTextRect.setY(tabTextRect.y() - padding);
+#endif
         QRect fontTextRect = fontMetrics().boundingRect(tab_title);
 
         if (fontTextRect.width() > tabTextRect.width())
             need_fade_out = true;
 
-        bool right_to_left = tab_title.isRightToLeft();
-
         if (need_fade_out) {
             // draw the most of tab text extent with the normal color,
             // and draw the rest with alpha channel gradient
@@ -433,7 +471,7 @@ void TabBar::paintEvent(QPaintEvent *e)
             const int mid_Y = tabTextRect.center().y();
             QLinearGradient gr;
 
-            if (right_to_left) {
+            if (textRightToLeft) {
                 // arabic right-to-left text
                 gr.setStart(tabTextRect.x(), mid_Y);
                 gr.setFinalStop(tabTextRect.x() + 0.2 * tabTextRect.width(), mid_Y);
@@ -452,7 +490,13 @@ void TabBar::paintEvent(QPaintEvent *e)
 
         int align = Qt::AlignVCenter;
         if (need_fade_out) {
-            align |= (right_to_left)? Qt::AlignRight : Qt::AlignLeft;
+            /* 
+               We align LTR text in LTR app to the left, while RTL text in RTL app
+               to the right. Since Qt mirror alignment, we can simply do ALignLeft
+               for both cases. When text and app direction doesn't match, we flip
+               the text alignment.
+            */
+            align |= (textRightToLeft == appRightToLeft) ? Qt::AlignLeft : Qt::AlignRight;
         }
 
         style()->drawItemText(&p, tabTextRect, align,
@@ -460,6 +504,24 @@ void TabBar::paintEvent(QPaintEvent *e)
     }
 }
 
+void TabBar::tabRemoved(int index)
+{
+    QTabBar::tabRemoved(index);
+    emit tabRemovedSignal(index);
+}
+
+void TabBar::tabInserted(int index)
+{
+    QTabBar::tabInserted(index);
+    emit tabInsertedSignal(index);
+}
+
+void TabBar::resizeEvent(QResizeEvent *event)
+{
+    QTabBar::resizeEvent(event);
+    emit sizeChanged();
+}
+
 void TabBar::onTabMoved(int from, int to)
 {
     // avoid infinitive recursion
@@ -484,4 +546,6 @@ void TabBar::onTabMoved(int from, int to)
     QWidget *w_from = mp_stackedWidget->widget(from);
     mp_stackedWidget->removeWidget(w_from);
     mp_stackedWidget->insertWidget(to, w_from);
+
+    KiwixApp::instance()->saveListOfOpenTabs();
 }
