@@ -5,6 +5,7 @@
 
 class UrlSchemeHandler : public QWebEngineUrlSchemeHandler
 {
+	Q_OBJECT
 public:
     UrlSchemeHandler();
     void requestStarted(QWebEngineUrlRequestJob *request);
@@ -12,6 +13,9 @@ private:
     void handleMetaRequest(QWebEngineUrlRequestJob *request);
     void handleContentRequest(QWebEngineUrlRequestJob *request);
     void handleSearchRequest(QWebEngineUrlRequestJob *request);
+
+    void replyZimNotFoundPage(QWebEngineUrlRequestJob *request, const QString& zimId);
+    void replyBadZimFilePage(QWebEngineUrlRequestJob *request, const QString& zimId);
 };
 
 #endif // URLSCHEMEHANDLER_H
