@@ -1,3 +1,5 @@
+class QMenu;
+
 #include "webview.h"
 
 #include <QDesktopServices>
@@ -5,12 +7,22 @@
 #include <iostream>
 #include "kiwixapp.h"
 #include "webpage.h"
+#include "css_constants.h"
 #include <QToolTip>
 #include <QWebEngineSettings>
 #include <QWebEngineHistory>
 #include <QVBoxLayout>
+#include <QFileDialog>
 #include <zim/error.h>
 #include <zim/item.h>
+#include <kiwix/tools.h>
+#include <QWebChannel>
+#include <QWebEngineScript>
+#include "kiwixwebchannelobject.h"
+#include "tableofcontentbar.h"
+
+zim::Entry getArchiveEntryFromUrl(const zim::Archive& archive, const QUrl& url);
+QString askForSaveFilePath(const QString& suggestedName);
 
 void WebViewBackMenu::showEvent(QShowEvent *)
 {
@@ -29,7 +41,9 @@ void WebViewBackMenu::showEvent(QShowEvent *)
      */
 
     QRect geo = geometry();
-    geo.moveLeft(geo.left() + 6);  // see also: style.css: QToolButton#backButton { margin-left: 6px; }
+
+    const int marginLeft = CSS::TopWidget::QToolButton::backButton::marginLeft;
+    geo.moveLeft(geo.left() + marginLeft);
     geo.moveTop(geo.top() + 2);
     setGeometry(geo);
 }
@@ -80,11 +94,27 @@ WebView::WebView(QWidget *parent)
      * If the page is search results, we put the default zoom factor
      * If in Qt 6.x, the bug is fixed this code can be removed.
      */
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(this, &QWebEngineView::loadFinished, this, [=] (bool ok) {
         if (ok) {
             applyCorrectZoomFactor();
         }
     });
+#endif
+
+    const auto channel = new QWebChannel(this);
+    const auto kiwixChannelObj = new KiwixWebChannelObject;
+    page()->setWebChannel(channel, QWebEngineScript::UserWorld);
+    channel->registerObject("kiwixChannelObj", kiwixChannelObj);
+    
+    const auto tabbar = KiwixApp::instance()->getTabWidget();
+    connect(tabbar, &TabBar::currentTitleChanged, this, &WebView::onCurrentTitleChanged);
+    connect(kiwixChannelObj, &KiwixWebChannelObject::headersChanged, this, &WebView::onHeadersReceived);
+
+    const auto tocbar = KiwixApp::instance()->getMainWindow()->getTableOfContentBar();
+    connect(this, &WebView::headersChanged, tocbar, &TableOfContentBar::setupTree);
+    connect(tocbar, &TableOfContentBar::navigationRequested, this, &WebView::onNavigationRequested);
+    connect(this, &WebView::navigationRequested, kiwixChannelObj, &KiwixWebChannelObject::navigationRequested);
 }
 
 WebView::~WebView()
@@ -127,7 +157,89 @@ QMenu* WebView::getHistoryForwardMenu() const
     return ret;
 }
 
-void WebView::addHistoryItemAction(QMenu *menu, const QWebEngineHistoryItem &item, int n) const
+namespace
+{
+
+/**
+ * @brief Get the Zim Item object corresponding to the given url.
+ * 
+ * @param url QUrl
+ * @return zim::Item
+ * 
+ * @exception throws exception if zimId is invalid, archive doesn't exist,
+ * entry is invalid or not found, or entry is redirect.
+ */
+zim::Item getZimItem(const QUrl& url)
+{
+    const auto app = KiwixApp::instance();
+    const auto library = app->getLibrary();
+    const auto archive = library->getArchive(getZimIdFromUrl(url));
+    const auto entry = getArchiveEntryFromUrl(*archive, url);
+    return entry.getItem(true);
+}
+
+bool isHTMLContent(const zim::Item& item)
+{
+    auto mimeType = QByteArray::fromStdString(item.getMimetype());
+    mimeType = mimeType.split(';')[0];
+    return mimeType == "text/html";
+}
+
+}
+
+void WebView::saveViewContent()
+{
+    try {
+        const auto item = getZimItem(url());
+
+        /* We have to sanitize here, as parsing will start once we pass the file
+           name to either save or download method.
+        */
+        const QString suggestedFileName = QString::fromStdString(kiwix::getSlugifiedFileName(item.getTitle()));
+        if (isHTMLContent(item))
+        {
+            const QString fileName = askForSaveFilePath(suggestedFileName + ".pdf");
+            if (!fileName.isEmpty())
+                page()->printToPdf(fileName);
+        }
+        else
+            page()->download(this->url(), suggestedFileName);
+    }
+    catch (...) { /* Blank */}
+}
+
+void WebView::onCurrentTitleChanged()
+{
+    const auto tabbar = KiwixApp::instance()->getTabWidget();
+    const auto noAnchorUrl = url().url(QUrl::RemoveFragment);
+    const auto headersValid = m_headers["url"].toString() == noAnchorUrl;
+
+    /* If headers not valid for this webview, then we are loading and the emit 
+       will be handled by KiwixWebChannelObject::headersChanged.
+    */
+    if (tabbar->currentWebView() == this && headersValid)
+        emit headersChanged(m_headers);
+}
+
+void WebView::onHeadersReceived(const QString& headersJSONStr)
+{
+    const auto tabbar = KiwixApp::instance()->getTabWidget();
+    m_headers = QJsonDocument::fromJson(headersJSONStr.toUtf8()).object();
+    
+    if (tabbar->currentWebView() == this)
+        emit headersChanged(m_headers);
+}
+
+void WebView::onNavigationRequested(const QString &url, const QString &anchor)
+{
+    const auto tabbar = KiwixApp::instance()->getTabWidget();
+    if (tabbar->currentWebView() == this)
+        emit navigationRequested(url, anchor);
+}
+
+void WebView::addHistoryItemAction(QMenu *menu,
+                                   const QWebEngineHistoryItem &item,
+                                   int n) const
 {
     QAction *a = menu->addAction(item.title());
     a->setData(QVariant::fromValue(n));
@@ -162,30 +274,17 @@ QWebEngineView* WebView::createWindow(QWebEnginePage::WebWindowType type)
 
 void WebView::onUrlChanged(const QUrl& url) {
     auto zimId = getZimIdFromUrl(url);
+    auto app = KiwixApp::instance();
+    app->saveListOfOpenTabs();
     if (m_currentZimId == zimId ) {
         return;
     }
     m_currentZimId = zimId;
     emit zimIdChanged(m_currentZimId);
-    auto app = KiwixApp::instance();
-    std::shared_ptr<zim::Archive> archive;
-    try {
-        archive = app->getLibrary()->getArchive(m_currentZimId);
-    } catch (std::out_of_range& e) {
-        return;
-    }
+    m_icon = app->getLibrary()->getBookIcon(m_currentZimId);
     auto zoomFactor = app->getSettingsManager()->getZoomFactorByZimId(zimId);
     this->setZoomFactor(zoomFactor);
-    try {
-        std::string favicon, _mimetype;
-        auto item = archive->getIllustrationItem(48);
-        favicon = item.getData();
-        _mimetype = item.getMimetype();
-        QPixmap pixmap;
-        pixmap.loadFromData((const uchar*)favicon.data(), favicon.size());
-        m_icon = QIcon(pixmap);
-        emit iconChanged(m_icon);
-    } catch (zim::EntryNotFound& e) {}
+    emit iconChanged(m_icon);
 }
 
 void WebView::wheelEvent(QWheelEvent *event) {
@@ -202,26 +301,79 @@ void WebView::wheelEvent(QWheelEvent *event) {
 
 void WebView::contextMenuEvent(QContextMenuEvent *event)
 {
-    auto menu = this->page()->createStandardContextMenu();
-    pageAction(QWebEnginePage::OpenLinkInNewWindow)->setVisible(false);
-    if (!m_linkHovered.isEmpty()) {
-        if (!m_linkHovered.startsWith("zim://")) {
-            pageAction(QWebEnginePage::OpenLinkInNewTab)->setVisible(false);
-            auto openLinkInWebBrowserAction = new QAction(gt("open-link-in-web-browser"));
-            menu->insertAction(pageAction(QWebEnginePage::DownloadLinkToDisk) , openLinkInWebBrowserAction);
-            connect(menu, &QObject::destroyed, openLinkInWebBrowserAction, &QObject::deleteLater);
-            connect(openLinkInWebBrowserAction, &QAction::triggered, this, [=](bool checked) {
-                Q_UNUSED(checked);
-                QDesktopServices::openUrl(m_linkHovered);
-            });
-        } else {
-            pageAction(QWebEnginePage::OpenLinkInNewTab)->setVisible(true);
-        }
+    QMenu* menu;
+    if (m_linkHovered.isEmpty()) {
+        menu = createStandardContextMenu();
+    } else {
+        menu = createLinkContextMenu();
     }
+
+#if defined(QT_TEXTTOSPEECH_LIB)
+    const auto app = KiwixApp::instance();
+    menu->addAction(app->getAction(KiwixApp::ReadArticleAction));
+    if (page()->hasSelection())
+        menu->addAction(app->getAction(KiwixApp::ReadTextAction));
+#endif
+
     menu->exec(event->globalPos());
 }
 
 
+QMenu* WebView::createStandardContextMenu() {
+    auto app = KiwixApp::instance();
+
+    QMenu* menu = new QMenu(this);
+    auto backAction = new QAction(gt("back"));
+    backAction->setEnabled(app->getAction(KiwixApp::HistoryBackAction)->isEnabled());
+    backAction->setIcon(app->getAction(KiwixApp::HistoryBackAction)->icon());
+    menu->addAction(backAction);
+    connect(menu, &QObject::destroyed, backAction, &QObject::deleteLater);
+    connect(backAction, &QAction::triggered, this, [=](bool checked) {
+        Q_UNUSED(checked);
+        KiwixApp::instance()->getTabWidget()->triggerWebPageAction(QWebEnginePage::Back);
+    });
+
+    auto forwardAction = new QAction(gt("forward"));
+    forwardAction->setEnabled(app->getAction(KiwixApp::HistoryForwardAction)->isEnabled());
+    forwardAction->setIcon(app->getAction(KiwixApp::HistoryForwardAction)->icon());
+    menu->addAction(forwardAction);
+    connect(menu, &QObject::destroyed, forwardAction, &QObject::deleteLater);
+    connect(forwardAction, &QAction::triggered, this, [=](bool checked) {
+        Q_UNUSED(checked);
+        KiwixApp::instance()->getTabWidget()->triggerWebPageAction(QWebEnginePage::Forward);
+    });
+
+    menu->addAction(app->getAction(KiwixApp::SavePageAsAction));
+    return menu;
+}
+
+
+QMenu* WebView::createLinkContextMenu() {
+    QMenu* menu = new QMenu(this);
+
+    if (!m_linkHovered.startsWith("zim://")) {
+        auto openLinkInWebBrowserAction = new QAction(gt("open-link-in-web-browser"));
+        menu->addAction(openLinkInWebBrowserAction);
+        connect(menu, &QObject::destroyed, openLinkInWebBrowserAction, &QObject::deleteLater);
+        connect(openLinkInWebBrowserAction, &QAction::triggered, this, [=](bool checked) {
+            Q_UNUSED(checked);
+            QDesktopServices::openUrl(m_linkHovered);
+        });
+    } else {
+        auto openLinkNewTab = new QAction(gt("open-link-new-tab"));
+        openLinkNewTab->setIcon(QIcon(":/icons/new-tab-icon.svg"));
+        menu->addAction(openLinkNewTab);
+        connect(menu, &QObject::destroyed, openLinkNewTab, &QObject::deleteLater);
+        connect(openLinkNewTab, &QAction::triggered, this, [=](bool checked) {
+            Q_UNUSED(checked);
+            KiwixApp::instance()->openUrl(m_linkHovered, true);
+        });
+    }
+
+    return menu;
+}
+
+
 bool WebView::eventFilter(QObject *src, QEvent *e)
 {
     Q_UNUSED(src)
