@@ -1,10 +1,11 @@
 #ifndef WEBVIEW_H
 #define WEBVIEW_H
 
+#include <QMenu>
 #include <QWebEngineView>
 #include <QIcon>
 #include <QWheelEvent>
-#include <QMenu>
+#include <QJsonObject>
 
 #include "findinpagebar.h"
 
@@ -45,12 +46,16 @@ public:
     QMenu* getHistoryBackMenu() const;
     QMenu* getHistoryForwardMenu() const;
 
+    void saveViewContent();
+
 public slots:
     void onUrlChanged(const QUrl& url);
 
 signals:
     void iconChanged(const QIcon& icon);
     void zimIdChanged(const QString& zimId);
+    void headersChanged(const QJsonObject& headers);
+    void navigationRequested(const QString& url, const QString& anchor);
 
 protected:
     virtual QWebEngineView* createWindow(QWebEnginePage::WebWindowType type);
@@ -65,10 +70,16 @@ protected:
 
 private slots:
     void gotoTriggeredHistoryItemAction();
+    void onCurrentTitleChanged();
+    void onHeadersReceived(const QString& headersJSONStr);
+    void onNavigationRequested(const QString& url, const QString& anchor);
 
 private:
     void addHistoryItemAction(QMenu *menu, const QWebEngineHistoryItem &item, int n) const;
     void applyCorrectZoomFactor();
+    QMenu* createStandardContextMenu();
+    QMenu* createLinkContextMenu();
+    QJsonObject m_headers;
 };
 
 #endif // WEBVIEW_H
